@@ -447,17 +447,10 @@ bool BoxedInstance::get_nested_interface_object(
         return false;
     }
 
-    JS::RootedObject proto(
-        context, gjs_lookup_generic_prototype(context, interface_info));
-
-    if (!proto)
-        return false;
-
     offset = g_field_info_get_offset (field_info);
 
-    JS::RootedObject obj(context, JS_NewObjectWithGivenProto(
-                                      context, JS_GetClass(proto), proto));
-
+    JS::RootedObject obj(context, gjs_new_object_with_generic_prototype(
+                                      context, interface_info));
     if (!obj)
         return false;
 
@@ -711,8 +704,6 @@ bool BoxedPrototype::define_boxed_class_fields(JSContext* cx,
 
 // Overrides GIWrapperPrototype::trace_impl().
 void BoxedPrototype::trace_impl(JSTracer* trc) {
-    JS::TraceEdge<jsid>(trc, &m_zero_args_constructor_name,
-                        "Boxed::zero_args_constructor_name");
     JS::TraceEdge<jsid>(trc, &m_default_constructor_name,
                         "Boxed::default_constructor_name");
     if (m_field_map)
@@ -868,7 +859,6 @@ struct_is_simple(GIStructInfo *info)
 BoxedPrototype::BoxedPrototype(GIStructInfo* info, GType gtype)
     : GIWrapperPrototype(info, gtype),
       m_zero_args_constructor(-1),
-      m_zero_args_constructor_name(JSID_VOID),
       m_default_constructor(-1),
       m_default_constructor_name(JSID_VOID),
       m_can_allocate_directly(struct_is_simple(info)) {
@@ -880,6 +870,7 @@ bool BoxedPrototype::init(JSContext* context) {
     int i, n_methods;
     int first_constructor = -1;
     jsid first_constructor_name = JSID_VOID;
+    jsid zero_args_constructor_name = JSID_VOID;
 
     if (m_gtype != G_TYPE_NONE) {
         /* If the structure is registered as a boxed, we can create a new instance by
@@ -907,9 +898,9 @@ bool BoxedPrototype::init(JSContext* context) {
                 if (m_zero_args_constructor < 0 &&
                     g_callable_info_get_n_args(func_info) == 0) {
                     m_zero_args_constructor = i;
-                    m_zero_args_constructor_name =
+                    zero_args_constructor_name =
                         gjs_intern_string_to_id(context, func_info.name());
-                    if (m_zero_args_constructor_name == JSID_VOID)
+                    if (zero_args_constructor_name == JSID_VOID)
                         return false;
                 }
 
@@ -924,7 +915,7 @@ bool BoxedPrototype::init(JSContext* context) {
 
         if (m_default_constructor < 0) {
             m_default_constructor = m_zero_args_constructor;
-            m_default_constructor_name = m_zero_args_constructor_name;
+            m_default_constructor_name = zero_args_constructor_name;
         }
         if (m_default_constructor < 0) {
             m_default_constructor = first_constructor;
@@ -963,12 +954,7 @@ JSObject* gjs_boxed_from_c_struct(JSContext* cx, GIStructInfo* info,
                       "Wrapping struct %s %p with JSObject",
                       g_base_info_get_name((GIBaseInfo *)info), gboxed);
 
-    JS::RootedObject proto(cx, gjs_lookup_generic_prototype(cx, info));
-    if (!proto)
-        return nullptr;
-
-    JS::RootedObject obj(
-        cx, JS_NewObjectWithGivenProto(cx, JS_GetClass(proto), proto));
+    JS::RootedObject obj(cx, gjs_new_object_with_generic_prototype(cx, info));
     if (!obj)
         return nullptr;
 
