@@ -342,6 +342,8 @@ void GjsContextPrivate::dispose(void) {
 
         JS_BeginRequest(m_cx);
 
+        gjs_debug(GJS_DEBUG_CONTEXT, "Releasing cached JS wrappers");
+        m_fundamental_table->clear();
         /* Do a full GC here before tearing down, since once we do
          * that we may not have the JS_GetPrivate() to access the
          * context
@@ -372,6 +374,7 @@ void GjsContextPrivate::dispose(void) {
 
         gjs_debug(GJS_DEBUG_CONTEXT, "Freeing allocated resources");
         delete m_job_queue;
+        delete m_fundamental_table;
 
         /* Tear down JS */
         JS_DestroyContext(m_cx);
@@ -451,6 +454,11 @@ GjsContextPrivate::GjsContextPrivate(JSContext* cx, GjsContext* public_context)
     if (!m_job_queue)
         g_error("Failed to initialize promise job queue");
 
+    JSRuntime* rt = JS_GetRuntime(m_cx);
+    m_fundamental_table = new JS::WeakCache<FundamentalTable>(rt);
+    if (!m_fundamental_table->init())
+        g_error("Failed to initialize fundamental objects table");
+
     JS_BeginRequest(m_cx);
 
     JS::RootedObject global(m_cx, gjs_create_global_object(m_cx));
