@@ -23,6 +23,7 @@
 #include <config.h>
 
 #include "gi/foreign.h"
+#include "gjs/context-private.h"
 #include "gjs/jsapi-class.h"
 #include "gjs/jsapi-util-args.h"
 #include "gjs/jsapi-wrapper.h"
@@ -37,6 +38,7 @@ typedef struct {
     cairo_region_t *region;
 } GjsCairoRegion;
 
+GJS_USE
 static JSObject *gjs_cairo_region_get_proto(JSContext *);
 
 GJS_DEFINE_PROTO_WITH_GTYPE("Region", cairo_region,
@@ -55,6 +57,7 @@ get_region(JSContext       *context,
         return priv->region;
 }
 
+GJS_JSAPI_RETURN_CONVENTION
 static bool
 fill_rectangle(JSContext             *context,
                JS::HandleObject       obj,
@@ -67,45 +70,40 @@ fill_rectangle(JSContext             *context,
 #define RETURN_STATUS                                           \
     return gjs_cairo_check_status(context, cairo_region_status(this_region), "region");
 
-#define REGION_DEFINE_REGION_FUNC(method)                       \
-    static bool                                                 \
-    method##_func(JSContext *context,                           \
-                  unsigned argc,                                \
-                  JS::Value *vp)                                \
-    {                                                           \
-        PRELUDE;                                                \
-        JS::RootedObject other_obj(context);                    \
-        cairo_region_t *other_region;                           \
-        if (!gjs_parse_call_args(context, #method, argv, "o",   \
-                                 "other_region", &other_obj))   \
-            return false;                                       \
-                                                                \
-        other_region = get_region(context, other_obj);          \
-                                                                \
-        cairo_region_##method(this_region, other_region);       \
-            argv.rval().setUndefined();                         \
-            RETURN_STATUS;                                      \
+#define REGION_DEFINE_REGION_FUNC(method)                                     \
+    GJS_JSAPI_RETURN_CONVENTION                                               \
+    static bool method##_func(JSContext* context, unsigned argc,              \
+                              JS::Value* vp) {                                \
+        PRELUDE;                                                              \
+        JS::RootedObject other_obj(context);                                  \
+        if (!gjs_parse_call_args(context, #method, argv, "o", "other_region", \
+                                 &other_obj))                                 \
+            return false;                                                     \
+                                                                              \
+        cairo_region_t* other_region = get_region(context, other_obj);        \
+                                                                              \
+        cairo_region_##method(this_region, other_region);                     \
+        argv.rval().setUndefined();                                           \
+        RETURN_STATUS;                                                        \
     }
 
-#define REGION_DEFINE_RECT_FUNC(method)                         \
-    static bool                                                 \
-    method##_rectangle_func(JSContext *context,                 \
-                            unsigned argc,                      \
-                            JS::Value *vp)                      \
-    {                                                           \
-        PRELUDE;                                                \
-        JS::RootedObject rect_obj(context);                     \
-        cairo_rectangle_int_t rect;                             \
-        if (!gjs_parse_call_args(context, #method, argv, "o",   \
-                                 "rect", &rect_obj))            \
-            return false;                                       \
-                                                                \
-        if (!fill_rectangle(context, rect_obj, &rect))          \
-            return false;                                       \
-                                                                \
-        cairo_region_##method##_rectangle(this_region, &rect);  \
-            argv.rval().setUndefined();                         \
-            RETURN_STATUS;                                      \
+#define REGION_DEFINE_RECT_FUNC(method)                                    \
+    GJS_JSAPI_RETURN_CONVENTION                                            \
+    static bool method##_rectangle_func(JSContext* context, unsigned argc, \
+                                        JS::Value* vp) {                   \
+        PRELUDE;                                                           \
+        JS::RootedObject rect_obj(context);                                \
+        if (!gjs_parse_call_args(context, #method, argv, "o", "rect",      \
+                                 &rect_obj))                               \
+            return false;                                                  \
+                                                                           \
+        cairo_rectangle_int_t rect;                                        \
+        if (!fill_rectangle(context, rect_obj, &rect))                     \
+            return false;                                                  \
+                                                                           \
+        cairo_region_##method##_rectangle(this_region, &rect);             \
+        argv.rval().setUndefined();                                        \
+        RETURN_STATUS;                                                     \
     }
 
 REGION_DEFINE_REGION_FUNC(union)
@@ -118,29 +116,31 @@ REGION_DEFINE_RECT_FUNC(subtract)
 REGION_DEFINE_RECT_FUNC(intersect)
 REGION_DEFINE_RECT_FUNC(xor)
 
+GJS_JSAPI_RETURN_CONVENTION
 static bool
 fill_rectangle(JSContext             *context,
                JS::HandleObject       obj,
                cairo_rectangle_int_t *rect)
 {
+    const GjsAtoms& atoms = GjsContextPrivate::atoms(context);
     JS::RootedValue val(context);
 
-    if (!gjs_object_get_property(context, obj, GJS_STRING_X, &val))
+    if (!JS_GetPropertyById(context, obj, atoms.x(), &val))
         return false;
     if (!JS::ToInt32(context, val, &rect->x))
         return false;
 
-    if (!gjs_object_get_property(context, obj, GJS_STRING_Y, &val))
+    if (!JS_GetPropertyById(context, obj, atoms.y(), &val))
         return false;
     if (!JS::ToInt32(context, val, &rect->y))
         return false;
 
-    if (!gjs_object_get_property(context, obj, GJS_STRING_WIDTH, &val))
+    if (!JS_GetPropertyById(context, obj, atoms.width(), &val))
         return false;
     if (!JS::ToInt32(context, val, &rect->width))
         return false;
 
-    if (!gjs_object_get_property(context, obj, GJS_STRING_HEIGHT, &val))
+    if (!JS_GetPropertyById(context, obj, atoms.height(), &val))
         return false;
     if (!JS::ToInt32(context, val, &rect->height))
         return false;
@@ -148,28 +148,37 @@ fill_rectangle(JSContext             *context,
     return true;
 }
 
+GJS_JSAPI_RETURN_CONVENTION
 static JSObject *
 make_rectangle(JSContext *context,
                cairo_rectangle_int_t *rect)
 {
+    const GjsAtoms& atoms = GjsContextPrivate::atoms(context);
     JS::RootedObject rect_obj(context, JS_NewPlainObject(context));
+    if (!rect_obj)
+        return nullptr;
     JS::RootedValue val(context);
 
     val = JS::Int32Value(rect->x);
-    JS_SetProperty(context, rect_obj, "x", val);
+    if (!JS_SetPropertyById(context, rect_obj, atoms.x(), val))
+        return nullptr;
 
     val = JS::Int32Value(rect->y);
-    JS_SetProperty(context, rect_obj, "y", val);
+    if (!JS_SetPropertyById(context, rect_obj, atoms.y(), val))
+        return nullptr;
 
     val = JS::Int32Value(rect->width);
-    JS_SetProperty(context, rect_obj, "width", val);
+    if (!JS_SetPropertyById(context, rect_obj, atoms.width(), val))
+        return nullptr;
 
     val = JS::Int32Value(rect->height);
-    JS_SetProperty(context, rect_obj, "height", val);
+    if (!JS_SetPropertyById(context, rect_obj, atoms.height(), val))
+        return nullptr;
 
     return rect_obj;
 }
 
+GJS_JSAPI_RETURN_CONVENTION
 static bool
 num_rectangles_func(JSContext *context,
                     unsigned argc,
@@ -186,6 +195,7 @@ num_rectangles_func(JSContext *context,
     RETURN_STATUS;
 }
 
+GJS_JSAPI_RETURN_CONVENTION
 static bool
 get_rectangle_func(JSContext *context,
                    unsigned argc,
@@ -278,6 +288,7 @@ gjs_cairo_region_finalize(JSFreeOp *fop,
     g_slice_free(GjsCairoRegion, priv);
 }
 
+GJS_JSAPI_RETURN_CONVENTION
 static JSObject *
 gjs_cairo_region_from_region(JSContext *context,
                              cairo_region_t *region)
@@ -293,6 +304,7 @@ gjs_cairo_region_from_region(JSContext *context,
     return object;
 }
 
+GJS_USE
 static bool
 region_to_g_argument(JSContext      *context,
                      JS::Value       value,
@@ -315,6 +327,7 @@ region_to_g_argument(JSContext      *context,
     return true;
 }
 
+GJS_JSAPI_RETURN_CONVENTION
 static bool
 region_from_g_argument(JSContext             *context,
                        JS::MutableHandleValue value_p,
